@@ -33,7 +33,7 @@
 
 #include "telnet_locl.h"
 
-RCSID("$Id: commands.c,v 1.39 1997/09/29 14:04:39 assar Exp $");
+RCSID("$Id: commands.c,v 1.54 1999/07/12 10:25:22 bg Exp $");
 
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 int tos = -1;
@@ -66,7 +66,8 @@ makeargv()
     margc = 0;
     cp = line;
     if (*cp == '!') {		/* Special case shell escape */
-	strcpy(saveline, line);	/* save for shell command */
+	/* save for shell command */
+	strcpy_truncate(saveline, line, sizeof(saveline));
 	*argp++ = "!";		/* No room in string to get this */
 	margc++;
 	cp++;
@@ -451,6 +452,19 @@ togdebug()
     return 1;
 }
 
+#if defined(KRB4) && defined(HAVE_KRB_DISABLE_DEBUG)
+#include <krb.h>
+
+static int
+togkrbdebug(void)
+{
+    if(krb_debug)
+	krb_enable_debug();
+    else
+	krb_disable_debug();
+    return 1;
+}
+#endif
 
 static int
 togcrlf()
@@ -661,6 +675,13 @@ static struct togglelist Togglelist[] = {
 	    togdebug,
 		&debug,
 		    "turn on socket level debugging" },
+#if defined(KRB4) && defined(HAVE_KRB_DISABLE_DEBUG)
+    { "krb_debug",
+      "kerberos 4 debugging",
+      togkrbdebug,
+      &krb_debug,
+      "turn on kerberos 4 debugging" },
+#endif
     { "netdata",
 	"printing of hexadecimal network data (debugging)",
 	    0,
@@ -1560,10 +1581,9 @@ env_init(void)
 
 		/* If this is not the full name, try to get it via DNS */
 		if (strchr(hbuf, '.') == 0) {
-			struct hostent *he = gethostbyname(hbuf);
-			if (he != 0)
-				strncpy(hbuf, he->h_name, 256);
-			hbuf[256] = '\0';
+			struct hostent *he = roken_gethostbyname(hbuf);
+			if (he != NULL)
+				strcpy_truncate(hbuf, he->h_name, 256);
 		}
 
 		asprintf (&cp, "%s%s", hbuf, cp2);
@@ -1602,7 +1622,7 @@ env_define(unsigned char *var, unsigned char *value)
 		if (ep->next)
 			ep->next->prev = ep;
 	}
-	ep->welldefined = opt_welldefined(var);
+	ep->welldefined = opt_welldefined((char *)var);
 	ep->export = 1;
 	ep->var = (unsigned char *)strdup((char *)var);
 	ep->value = (unsigned char *)strdup((char *)value);
@@ -1939,7 +1959,7 @@ status(int argc, char **argv)
  * Function that gets called when SIGINFO is received.
  */
 void
-ayt_status(void)
+ayt_status(int ignore)
 {
     call(status, "status", "notmuch", 0);
 }
@@ -1961,7 +1981,7 @@ cmdrc(char *m1, char *m2)
     if (skiprc)
 	return;
 
-    strcpy(m1save, m1);
+    strcpy_truncate(m1save, m1, sizeof(m1save));
     m1 = m1save;
 
     if (rcname[0] == 0) {
@@ -2029,21 +2049,23 @@ int
 tn(int argc, char **argv)
 {
     struct hostent *host = 0;
-#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+#ifdef HAVE_IPV6
     struct sockaddr_in6 sin6;
 #endif
     struct sockaddr_in sin;
-    struct sockaddr *sa;
-    int sa_size;
+    struct sockaddr *sa = NULL;
+    int sa_size = 0;
     struct servent *sp = 0;
     unsigned long temp;
     extern char *inet_ntoa();
-#if	defined(IP_OPTIONS) && defined(IPPROTO_IP)
+#if defined(IP_OPTIONS) && defined(IPPROTO_IP)
     char *srp = 0;
     int srlen;
 #endif
-    char *cmd, *hostp = 0, *portp = 0, *user = 0;
-    int family, port;
+    char *cmd, *hostp = 0, *portp = 0;
+    char *user = 0;
+    int family, port = 0;
+    char **addr_list;
 
     /* clear the socket address prior to use */
 
@@ -2053,7 +2075,7 @@ tn(int argc, char **argv)
 	return 0;
     }
     if (argc < 2) {
-	strcpy(line, "open ");
+	strcpy_truncate(line, "open ", sizeof(line));
 	printf("(to) ");
 	fgets(&line[strlen(line)], sizeof(line) - strlen(line), stdin);
 	makeargv();
@@ -2069,7 +2091,7 @@ tn(int argc, char **argv)
 	    --argc; ++argv;
 	    if (argc == 0)
 		goto usage;
-	    user = *argv++;
+	    user = strdup(*argv++);
 	    --argc;
 	    continue;
 	}
@@ -2104,11 +2126,7 @@ tn(int argc, char **argv)
 	srp = 0;
 	temp = sourceroute(hostp, &srp, &srlen);
 	if (temp == 0) {
-#ifdef HAVE_H_ERRNO
 	    fprintf (stderr, "%s: %s\r\n", srp ? srp : "", hstrerror(h_errno));
-#else
-	    fprintf (stderr, "%s: %s\r\n", srp ? srp : "", "unknown error");
-#endif
 	    setuid(getuid());
 	    return 0;
 	} else if (temp == -1) {
@@ -2121,30 +2139,37 @@ tn(int argc, char **argv)
     } else {
 #endif
 	memset (&sin, 0, sizeof(sin));
-#if defined(HAVE_INET_PTON) && defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+#ifdef HAVE_IPV6
 	memset (&sin6, 0, sizeof(sin6));
 
 	if(inet_pton(AF_INET6, hostp, &sin6.sin6_addr)) {
 	    sin6.sin6_family = family = AF_INET6;
-	    strcpy(_hostname, hostp);
+	    sa = (struct sockaddr *)&sin6;
+	    sa_size = sizeof(sin6);
+	    strcpy_truncate(_hostname, hostp, sizeof(_hostname));
 	    hostname =_hostname;
 	} else
 #endif
 	if(inet_aton(hostp, &sin.sin_addr)){
 	    sin.sin_family = family = AF_INET;
-	    strcpy(_hostname, hostp);
+	    sa = (struct sockaddr *)&sin;
+	    sa_size = sizeof(sin);
+	    strcpy_truncate(_hostname, hostp, sizeof(_hostname));
 	    hostname = _hostname;
 	} else {
 #ifdef HAVE_GETHOSTBYNAME2
+#ifdef HAVE_IPV6
 	    host = gethostbyname2(hostp, AF_INET6);
 	    if(host == NULL)
+#endif
 		host = gethostbyname2(hostp, AF_INET);
 #else
-	    host = gethostbyname(hostp);
+	    host = roken_gethostbyname(hostp);
 #endif
 	    if (host) {
-		strncpy(_hostname, host->h_name, sizeof(_hostname));
+		strcpy_truncate(_hostname, host->h_name, sizeof(_hostname));
 		family = host->h_addrtype;
+		addr_list = host->h_addr_list;
 
 		switch(family) {
 		case AF_INET:
@@ -2152,15 +2177,15 @@ tn(int argc, char **argv)
 		    sa_size = sizeof(sin);
 		    sa = (struct sockaddr *)&sin;
 		    sin.sin_family = family;
-		    sin.sin_addr   = *((struct in_addr *)(*host->h_addr_list));
+		    sin.sin_addr   = *((struct in_addr *)(*addr_list));
 		    break;
-#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+#ifdef HAVE_IPV6
 		case AF_INET6:
 		    memset(&sin6, 0, sizeof(sin6));
 		    sa_size = sizeof(sin6);
 		    sa = (struct sockaddr *)&sin6;
 		    sin6.sin6_family = family;
-		    sin6.sin6_addr   = *((struct in6_addr *)(*host->h_addr_list));
+		    sin6.sin6_addr   = *((struct in6_addr *)(*addr_list));
 		    break;
 #endif
 		default:
@@ -2172,11 +2197,7 @@ tn(int argc, char **argv)
 		hostname = _hostname;
 	    } else {
 	        fprintf (stderr, "%s: %s\r\n", hostp ? hostp : "",
-#ifdef HAVE_H_ERRNO
 			 hstrerror(h_errno));
-#else
-			 "unknown error");
-#endif
 		setuid(getuid());
 		return 0;
 	    }
@@ -2192,7 +2213,7 @@ tn(int argc, char **argv)
 	    telnetport = 0;
 	port = atoi(portp);
 	if (port == 0) {
-	    sp = getservbyname(portp, "tcp");
+	    sp = roken_getservbyname(portp, "tcp");
 	    if (sp)
 		port = sp->s_port;
 	    else {
@@ -2205,7 +2226,7 @@ tn(int argc, char **argv)
 	}
     } else {
 	if (sp == 0) {
-	    sp = getservbyname("telnet", "tcp");
+	    sp = roken_getservbyname("telnet", "tcp");
 	    if (sp == 0) {
 		fprintf(stderr, "telnet: tcp/telnet: unknown service\r\n");
 		setuid(getuid());
@@ -2215,34 +2236,35 @@ tn(int argc, char **argv)
 	}
 	telnetport = 1;
     }
-    switch(family) {
-    case AF_INET:
-	sin.sin_port = port;
-	printf("Trying %s...\r\n", inet_ntoa(sin.sin_addr));
-	break;
-#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
-    case AF_INET6: {
+    do {
+	switch(family) {
+	case AF_INET:
+	    sin.sin_port = port;
+	    printf("Trying %s...\r\n", inet_ntoa(sin.sin_addr));
+	    break;
+#ifdef HAVE_IPV6
+	case AF_INET6: {
 #ifndef INET6_ADDRSTRLEN
 #define INET6_ADDRSTRLEN 46 
 #endif
 
-	char buf[INET6_ADDRSTRLEN];
+	    char buf[INET6_ADDRSTRLEN];
 
-	sin6.sin6_port = port;
+	    sin6.sin6_port = port;
 #ifdef HAVE_INET_NTOP
-	printf("Trying %s...\r\n", inet_ntop(AF_INET6,
-					     &sin6.sin6_addr,
-					     buf,
-					     sizeof(buf)));
+	    printf("Trying %s...\r\n", inet_ntop(AF_INET6,
+						 &sin6.sin6_addr,
+						 buf,
+						 sizeof(buf)));
 #endif
-	break;
-    }
+	    break;
+	}
 #endif
-    default:
-	abort();
-    }
+	default:
+	    abort();
+	}
+
 
-    do {
 	net = socket(family, SOCK_STREAM, 0);
 	setuid(getuid());
 	if (net < 0) {
@@ -2256,7 +2278,7 @@ tn(int argc, char **argv)
 #endif
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 	{
-# if	defined(HAS_GETTOS)
+# if	defined(HAVE_GETTOSBYNAME)
 	    struct tosent *tp;
 	    if (tos < 0 && (tp = gettosbyname("telnet", "tcp")))
 		tos = tp->t_tos;
@@ -2276,23 +2298,23 @@ tn(int argc, char **argv)
 	}
 
 	if (connect(net, sa, sa_size) < 0) {
-	    if (host && host->h_addr_list[1]) {
+	    if (host && addr_list[1]) {
 		int oerrno = errno;
 
 		switch(family) {
 		case AF_INET :
 		    fprintf(stderr, "telnet: connect to address %s: ",
 			    inet_ntoa(sin.sin_addr));
-		    sin.sin_addr = *((struct in_addr *)(*++host->h_addr_list));
+		    sin.sin_addr = *((struct in_addr *)(*++addr_list));
 		    break;
-#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
+#ifdef HAVE_IPV6
 		case AF_INET6: {
 		    char buf[INET6_ADDRSTRLEN];
 
 		    fprintf(stderr, "telnet: connect to address %s: ",
 			    inet_ntop(AF_INET6, &sin6.sin6_addr, buf,
 				      sizeof(buf)));
-		    sin6.sin6_addr = *((struct in6_addr *)(*++host->h_addr_list));
+		    sin6.sin6_addr = *((struct in6_addr *)(*++addr_list));
 		    break;
 		}
 #endif
@@ -2317,9 +2339,9 @@ tn(int argc, char **argv)
     if (autologin && user == NULL) {
 	struct passwd *pw;
 
-	user = getenv("USER");
+	user = (char *)get_default_username();
 	if (user == NULL ||
-	    ((pw = k_getpwnam(user)) && pw->pw_uid != getuid())) {
+	    ((pw = k_getpwnam((char *)user)) && pw->pw_uid != getuid())) {
 		if ((pw = k_getpwuid(getuid())))
 			user = pw->pw_name;
 		else
@@ -2332,14 +2354,14 @@ tn(int argc, char **argv)
     }
     call(status, "status", "notmuch", 0);
     if (setjmp(peerdied) == 0)
-	telnet(user);
+	my_telnet((char *)user);
     NetClose(net);
     ExitString("Connection closed by foreign host.\r\n",1);
     /*NOTREACHED*/
     return 0;
 }
 
-#define HELPINDENT (sizeof ("connect"))
+#define HELPINDENT ((int)sizeof ("connect"))
 
 static char
 	openhelp[] =	"connect to a site",
@@ -2649,15 +2671,10 @@ sourceroute(char *arg, char **cpp, int *lenp)
 
 		if ((tmp = inet_addr(cp)) != -1) {
 			sin_addr.s_addr = tmp;
-		} else if ((host = gethostbyname(cp))) {
-#if	defined(h_addr)
+		} else if ((host = roken_gethostbyname(cp))) {
 			memmove(&sin_addr,
 				host->h_addr_list[0],
 				sizeof(sin_addr));
-#else
-			memmove(&sin_addr, host->h_addr,
-				sizeof(sin_addr));
-#endif
 		} else {
 			*cpp = cp;
 			return(0);
